@@ -1,17 +1,4 @@
-/* The GPL applies to this program.
-  In addition, as a special exception, the copyright holders give
-  permission to link the code of portions of this program with the
-  OpenSSL library under certain conditions as described in each
-  individual source file, and distribute linked combinations
-  including the two.
-  You must obey the GNU General Public License in all respects
-  for all of the code used other than OpenSSL.  If you modify
-  file(s) with this exception, you may extend this exception to your
-  version of the file(s), but you are not obligated to do so.  If you
-  do not wish to do so, delete this exception statement from your
-  version.  If you delete this exception statement from all source
-  files in the program, then also delete it here.
-*/
+/* Released under GPLv2 with exception for the OpenSSL library. See license.txt */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -85,7 +72,7 @@ void help_long(void)
 	fprintf(stderr, "--ok-result-codes	-o\n");
 	fprintf(stderr, "--result-string	-e\n");
 	fprintf(stderr, "--user-agent		-I\n");
-	fprintf(stderr, "--referer		-S\n");
+	fprintf(stderr, "--referer		-R\n");
 	fprintf(stderr, "--resolve-once		-r\n");
 	fprintf(stderr, "--nagios-mode-1	-n\n");
 	fprintf(stderr, "--nagios-mode-2	-n\n");
@@ -142,6 +129,7 @@ void usage(void)
 	fprintf(stderr, "               pinging roundrobin DNS: also takes the first\n");
 	fprintf(stderr, "               DNS lookup out of the loop so that the first\n");
 	fprintf(stderr, "               measurement is also correct)\n");
+	fprintf(stderr, "-W             do not abort the program if resolving failed: keep retrying\n");
 	fprintf(stderr, "-n warn,crit   Nagios-mode: return 1 when avg. response time\n");
 	fprintf(stderr, "               >= warn, 2 if >= crit, otherwhise return 0\n");
 	fprintf(stderr, "-N x           Nagios mode 2: return 0 when all fine, 'x'\n");
@@ -149,8 +137,9 @@ void usage(void)
 	fprintf(stderr, "-y ip[:port]   bind to ip-address (and thus interface) [/port]\n");
 	fprintf(stderr, "-q             quiet, only returncode\n");
 	fprintf(stderr, "-A             Activate Basic authentication\n");
-	fprintf(stderr, "-U Username    needed for authentication\n");
-	fprintf(stderr, "-P Password    needed for authentication\n");
+	fprintf(stderr, "-U username    needed for authentication\n");
+	fprintf(stderr, "-P password    needed for authentication\n");
+	fprintf(stderr, "-T x           read the password fom the file 'x' (replacement for -P)\n");
 	fprintf(stderr, "-C cookie=value Add a cookie to the request\n");
 	fprintf(stderr, "-V             show the version\n\n");
 	fprintf(stderr, "\n");
@@ -218,7 +207,26 @@ int enc_b64(char *source, size_t source_lenght, char *target)
 	target[0] = 0;
 	return 1;
 } 
-/* Base64 encoding END */  
+/* Base64 encoding END */
+
+const char * read_file(const char *file)
+{
+	char buffer[4096] = { 0 }, *lf = NULL;
+	FILE *fh = fopen(file, "rb");
+	if (!fh)
+		error_exit("Cannot open password-file %s", file);
+
+	if (!fgets(buffer, sizeof buffer, fh))
+		error_exit("Problem reading password from file %s", file);
+
+	fclose(fh);
+
+	lf = strchr(buffer, '\n');
+	if (lf)
+		*lf = 0x00;
+
+	return strdup(buffer);
+}
 
 int main(int argc, char *argv[])
 {
@@ -242,8 +250,8 @@ int main(int argc, char *argv[])
 	char *useragent = NULL;
 	char *referer = NULL;
 	char *host = NULL;
-	char *pwd = NULL;
-	char *usr = NULL;
+	const char *pwd = NULL;
+	const char *usr = NULL;
 	char *cookie = NULL;
 	int port = 0;
 	char resolve_once = 0;
@@ -272,6 +280,7 @@ int main(int argc, char *argv[])
 	char no_cache = 0;
 	char *getcopyorg = NULL;
 	char tfo = 0;
+	char abort_on_resolve_failure = 1;
 
 	static struct option long_options[] =
 	{
@@ -324,10 +333,18 @@ int main(int argc, char *argv[])
 
 	buffer = (char *)mymalloc(page_size, "receive buffer");
 
-	while((c = getopt_long(argc, argv, "JZQ6Sy:XL:bBg:h:p:c:i:Gx:t:o:e:falqsmV?I:R:rn:N:z:AP:U:C:F", long_options, NULL)) != -1)
+	while((c = getopt_long(argc, argv, "WT:JZQ6Sy:XL:bBg:h:p:c:i:Gx:t:o:e:falqsmV?I:R:rn:N:z:AP:U:C:F", long_options, NULL)) != -1)
 	{
 		switch(c)
 		{
+			case 'W':
+				abort_on_resolve_failure = 0;
+				break;
+
+			case 'T':
+				pwd = read_file(optarg);
+				break;
+
 			case 'J':
 				help_long();
 				return 0;
@@ -641,7 +658,7 @@ int main(int argc, char *argv[])
 		client_ctx = initialize_ctx();
 		if (!client_ctx)
 		{
-			snprintf(last_error, ERROR_BUFFER_SIZE, "problem creating SSL context\n");
+			snprintf(last_error, sizeof last_error, "problem creating SSL context\n");
 			goto error_exit;
 		}
 	}
@@ -723,11 +740,28 @@ int main(int argc, char *argv[])
 		{
 			err++;
 			emit_error();
-			have_resolved = 1;
+			have_resolved = 0;
+			if (abort_on_resolve_failure)
+				error_exit(last_error);
 		}
 
 		ai_use = select_resolved_host(ai, use_ipv6);
-		get_addr(ai_use, &addr);
+		if (!ai_use)
+		{
+			snprintf(last_error, sizeof last_error, "No valid IPv4 or IPv6 address found for %s\n", host);
+
+			if (abort_on_resolve_failure)
+				error_exit(last_error);
+
+			// do not emit the resolve-error here: as 'have_resolved' is set to 0
+			// next, the program will try to resolve again anyway
+			// this prevents a double error-message while err is increased only
+			// once
+			have_resolved = 0;
+		}
+
+		if (have_resolved)
+			get_addr(ai_use, &addr);
 	}
 
 	if (persistent_connections)
@@ -770,21 +804,37 @@ persistent_loop:
 				{
 					err++;
 					emit_error();
+
+					if (abort_on_resolve_failure)
+						error_exit(last_error);
 					break;
 				}
 
 				ai_use = select_resolved_host(ai, use_ipv6);
+				if (!ai_use)
+				{
+					snprintf(last_error, sizeof last_error, "No valid IPv4 or IPv6 address found for %s\n", host);
+					emit_error();
+					err++;
+
+					if (abort_on_resolve_failure)
+						error_exit(last_error);
+
+					break;
+				}
+
 				get_addr(ai_use, &addr);
 
 				have_resolved = 1;
 			}
 
 			req_sent = 0;
-			
-			if ((persistent_connections && fd < 0) || (!persistent_connections))
+
+			if ((persistent_connections && fd < 0) || !persistent_connections)
 			{
 				fd = connect_to((struct sockaddr *)(bind_to_valid?bind_to:NULL), ai, timeout, &tfo, request, req_len, &req_sent);
 			}
+
 			if (fd == RC_CTRLC)	/* ^C pressed */
 				break;
 
@@ -811,7 +861,7 @@ persistent_loop:
 					fd = -1;
 					break;
 				}
-					
+
 #ifndef NO_SSL
 				if (use_ssl && ssl_h == NULL)
 				{
@@ -839,7 +889,7 @@ persistent_loop:
 			if (fd < 0)
 			{
 				if (fd == RC_TIMEOUT)
-					snprintf(last_error, ERROR_BUFFER_SIZE, "timeout connecting to host\n");
+					snprintf(last_error, sizeof last_error, "timeout connecting to host\n");
 
 				emit_error();
 				err++;
@@ -875,13 +925,13 @@ persistent_loop:
 				}
 
 				if (rc == -1)
-					snprintf(last_error, ERROR_BUFFER_SIZE, "error sending request to host\n");
+					snprintf(last_error, sizeof last_error, "error sending request to host\n");
 				else if (rc == RC_TIMEOUT)
-					snprintf(last_error, ERROR_BUFFER_SIZE, "timeout sending to host\n");
+					snprintf(last_error, sizeof last_error, "timeout sending to host\n");
 				else if (rc == RC_CTRLC)
 				{/* ^C */}
 				else if (rc == 0)
-					snprintf(last_error, ERROR_BUFFER_SIZE, "connection prematurely closed by peer\n");
+					snprintf(last_error, sizeof last_error, "connection prematurely closed by peer\n");
 
 				emit_error();
 
@@ -937,7 +987,7 @@ persistent_loop:
 				char *length = strstr(reply, "\nContent-Length:");
 				if (!length)
 				{
-					snprintf(last_error, ERROR_BUFFER_SIZE, "'Content-Length'-header missing!\n");
+					snprintf(last_error, sizeof last_error, "'Content-Length'-header missing!\n");
 					emit_error();
 					close(fd);
 					fd = -1;
@@ -968,9 +1018,9 @@ persistent_loop:
 				}
 
 				if (rc == RC_SHORTREAD)
-					snprintf(last_error, ERROR_BUFFER_SIZE, "error receiving reply from host\n");
+					snprintf(last_error, sizeof last_error, "short read during receiving reply-headers from host\n");
 				else if (rc == RC_TIMEOUT)
-					snprintf(last_error, ERROR_BUFFER_SIZE, "timeout receiving reply from host\n");
+					snprintf(last_error, sizeof last_error, "timeout while receiving reply-headers from host\n");
 
 				emit_error();
 
@@ -1033,7 +1083,7 @@ persistent_loop:
 
 				if (close_ssl_connection(ssl_h, fd) == -1)
 				{
-					snprintf(last_error, ERROR_BUFFER_SIZE, "error shutting down ssl\n");
+					snprintf(last_error, sizeof last_error, "error shutting down ssl\n");
 					emit_error();
 				}
 
