@@ -1,17 +1,4 @@
-/* The GPL applies to this program.
-  In addition, as a special exception, the copyright holders give
-  permission to link the code of portions of this program with the
-  OpenSSL library under certain conditions as described in each
-  individual source file, and distribute linked combinations
-  including the two.
-  You must obey the GNU General Public License in all respects
-  for all of the code used other than OpenSSL.  If you modify
-  file(s) with this exception, you may extend this exception to your
-  version of the file(s), but you are not obligated to do so.  If you
-  do not wish to do so, delete this exception statement from your
-  version.  If you delete this exception statement from all source
-  files in the program, then also delete it here.
-*/
+/* Released under GPLv2 with exception for the OpenSSL library. See license.txt */
 
 #include <errno.h>
 #include <string.h>
@@ -23,9 +10,9 @@
 #include <openssl/x509.h>
 #include <openssl/md5.h>
 
+#include "gen.h"
 #include "mssl.h"
 
-extern char last_error[];
 BIO *bio_err=0;
 
 char close_ssl_connection(SSL *ssl_h, int socket_h)
